local mvec3_copy = mvector3.copy

Hooks:PostHook(PlayerManager,"check_skills","playermanager_checkskills",function(self)
	
	local td = tweak_data.upgrades.values.player.defiance
	local delayedcalls = DelayedCallsFix or DelayedCalls
	
	if self:has_category_upgrade("player","defiance_last_chance") then 
		Hooks:Add("defianceperkdeck_on_player_armor_broken","defianceperkdeck_on_proc_last_chance",
			function()
				if not self:has_active_temporary_property("defiance_damage_bonus_cooldown") then 
					self:activate_temporary_property("defiance_damage_bonus_cooldown",td.last_chance.cooldown,true)
					self:activate_temporary_property("defiance_damage_bonus_mul",td.last_chance.duration,td.last_chance.damage_bonus_mul)
				end
			end
		)
	else
		Hooks:Remove("defianceperkdeck_on_proc_last_chance")
	end
	
	local function is_weapon_from_local_player(wpn_unit)
		if alive(wpn_unit) then 
			local wpn_base = wpn_unit:base()
			if wpn_base and not wpn_base.sentry_gun and wpn_base._setup and wpn_base._setup.user_unit == self:local_player() then 
				return true
			end
		end
		return false
	end
	
	if self:has_category_upgrade("player","defiance_electrocutioner") then 
		self._message_system:register(Message.OnEnemyKilled, "defianceperkdeck_on_proc_electrocutioner_kill",
			function(equipped_unit,variant,killed_unit)
				if alive(killed_unit) then 
					local dmg_ext = killed_unit:character_damage()
					if dmg_ext._defiance_electrocuted then
						if is_weapon_from_local_player(equipped_unit) then
							--restore health
							local player = self:local_player()
							if alive(player) then 
								local pcd = player:character_damage()
								pcd:change_health(pcd:_max_health() * td.electrocutioner.health_regen)
							end
							
							--give reload speed bonus
							self:set_property("defiance_electrocutioner_reload_speed_bonus",td.electrocutioner.reload_speed_bonus_mul)
						end
					end
				end
			end
		)
		self._message_system:register(Message.OnEnemyShot, "defianceperkdeck_on_proc_electrocutioner",
			function(unit,attack_data)
				local result = attack_data.result and attack_data.result.type
				local attacker_unit = attack_data.attacker_unit
				local variant = attack_data.variant
				if attacker_unit == self:player_unit() and variant == "bullet" then
					if alive(unit) then 
						local dmg_ext = unit:character_damage()
						if dmg_ext then 
							local ub = unit:base()
							if not (dmg_ext:dead() or ub.sentry_gun) then 
								if not self:has_active_temporary_property("defiance_electrocutioner_cooldown") then 
									self:activate_temporary_property("defiance_electrocutioner_cooldown",td.electrocutioner.cooldown,true)

									--[[
									if unit:brain() and unit:brain().action_request then 
										unit:brain():action_request({
											type = "idle",
											body_part = 1,
											non_persistent = true
										})
									end
									--]]
									
									local dir = Vector3()
									if attack_data.col_ray then
										dir = attack_data.col_ray.ray or dir
									end
									local pos = Vector3()
									if attack_data.pos then 
										pos = mvec3_copy(attack_data.pos)
									end

									local tase_data = {
										damage = 0,
										variant = "melee",
										pos = pos,
										attack_dir = dir,
										result = {
											variant = "melee",
											type = "taser_tased"
										}
									}
									
									local stun_duration = td.electrocutioner.stun_duration
									dmg_ext._tased_time = stun_duration

									dmg_ext._defiance_electrocuted = true
									--set a flag so that killing this enemy in the future will bestow reload speed/hp regen effects
									--killed_unit:anim_data() just indicates that they are mid-hurt, not the hurt variant, so i can't use that to check

									delayedcalls:Add("defiance_electrocutioner_" .. tostring(unit:key()),stun_duration,function()
										if alive(unit) then 
											local _dmg_ext = unit:character_damage()
											if _dmg_ext then 
												_dmg_ext._defiance_electrocuted = false
											end
										end
									end)
									
									dmg_ext:_call_listeners(tase_data)
								end
							end
						end
					end
				end
			end
		)
	else
		self._message_system:unregister(Message.OnEnemyShot, "defianceperkdeck_on_proc_electrocutioner")
		self._message_system:unregister(Message.OnEnemyKilled, "defianceperkdeck_on_proc_electrocutioner_kill")
	end
	
	if self:has_category_upgrade("player","defiance_killing_spree") then 
		self._message_system:register(Message.OnEnemyKilled, "defianceperkdeck_on_proc_killingspree",
			function(equipped_unit,variant,killed_unit)
				if alive(equipped_unit) then 
					if is_weapon_from_local_player(equipped_unit) then 
						local stacks = math.min(self:get_temporary_property("defiance_killing_spree_stacks",0) + 1,td.killing_spree.max_stacks)
						self:activate_temporary_property("defiance_killing_spree_stacks",td.killing_spree.multikill_expire_threshold,stacks)
						
						self:set_damage_absorption("defiance_killing_spree_absorption",stacks * td.killing_spree.damage_absorption)	
						managers.hud:set_info_meter(nil, {max = 100, total = 100, current = stacks * td.killing_spree.damage_resist_mul * 100})
						delayedcalls:Add("defiance_killing_spree_decay",td.killing_spree.multikill_expire_threshold,
							function () --reset stacks
								self:set_damage_absorption("defiance_killing_spree_absorption",0)
								managers.hud:set_info_meter(nil, {max = 100, total = 100, current = 0})
							end
						)
					end
				end
			end
		)
	else
		self._message_system:unregister(Message.OnEnemyKilled, "defianceperkdeck_on_proc_killingspree")
	end
end)

--this function is incompatible with other mods that change it
local defianceperkdeck_orig_body_armor_addend = PlayerManager.body_armor_skill_addend
PlayerManager.defianceperkdeck_orig_body_armor_addend = defianceperkdeck_orig_body_armor_addend
function PlayerManager:body_armor_skill_addend(override_armor,...)
--	local addend = self:defianceperkdeck_orig_body_armor_addend(override_armor,...)
	local addend = 0
	addend = addend + self:upgrade_value("player", tostring(override_armor or managers.blackmarket:equipped_armor(true, true)) .. "_armor_addend", 0)
	
	if managers.player:has_category_upgrade("player","defiance_sentinel") then 
		local health_multiplier = self:defianceperkdeck_orig_health_mul()
		local max_health = (PlayerDamage._HEALTH_INIT + self:health_skill_addend()) * health_multiplier
		
		addend = addend + ((max_health * tweak_data.upgrades.values.player.defiance.sentinel.health_converted) * tweak_data.upgrades.values.player.defiance.sentinel.health_to_armor_conversion_rate)
	end
	
	if self:has_category_upgrade("player", "armor_increase") then
		local health_multiplier = self:health_skill_multiplier()
		local max_health = (PlayerDamage._HEALTH_INIT + self:health_skill_addend()) * health_multiplier
		addend = addend + max_health * self:upgrade_value("player", "armor_increase", 1)
	end

	addend = addend + self:upgrade_value("team", "crew_add_armor", 0)

	return addend
end



local defianceperkdeck_orig_movement_speed_multiplier = PlayerManager.movement_speed_multiplier
PlayerManager.defianceperkdeck_orig_movement_speed_multiplier = defianceperkdeck_orig_movement_speed_multiplier
function PlayerManager:movement_speed_multiplier(...)
	local result = defianceperkdeck_orig_movement_speed_multiplier(self,...) + self:get_temporary_property("defiance_move_speed_bonus",0)
	return result
end

local defianceperkdeck_orig_damage_resist = PlayerManager.damage_reduction_skill_multiplier
PlayerManager.defianceperkdeck_orig_damage_resist = defianceperkdeck_orig_damage_resist
function PlayerManager:damage_reduction_skill_multiplier(...)
	local result = defianceperkdeck_orig_damage_resist(self,...)
	return result * (1 - (tweak_data.upgrades.values.player.defiance.killing_spree.damage_resist_mul * self:get_temporary_property("defiance_killing_spree_stacks",0)))
end

local defianceperkdeck_orig_health_mul = PlayerManager.health_skill_multiplier
PlayerManager.defianceperkdeck_orig_health_mul = defianceperkdeck_orig_health_mul
function PlayerManager:health_skill_multiplier(...)
	local result = defianceperkdeck_orig_health_mul(self,...)
	if self:has_category_upgrade("player","defiance_sentinel") then 
		result = result * (1 - tweak_data.upgrades.values.player.defiance.sentinel.health_converted)
	end
	return result
end
